@@ -13,12 +13,15 @@ from .db import DatabaseLayer
 
 
 if sys.version_info < (3, 7):
-    create_task = asyncio.get_event_loop().create_task
+    create_task = asyncio.ensure_future
 else:
     from asyncio import create_task
 
+# Create two connection pools
+# short_pool is for short lived connections (i.e for sending, flushing and discarding)
 logger = logging.getLogger(__name__)
 pool = None
+short_pool = None
 creating_pool = False
 
 
@@ -38,7 +41,8 @@ class PostgresChannelLayer(BaseChannelLayer):
 
     def __init__(
         self, prefix='asgi', expiry=60, group_expiry=0,
-        symmetric_encryption_keys=None, **kwargs
+        symmetric_encryption_keys=None, config=dict(),
+        **kwargs
     ):
         self.prefix = prefix
         self.expiry = expiry
@@ -46,6 +50,8 @@ class PostgresChannelLayer(BaseChannelLayer):
         self.client_prefix = uuid.uuid4().hex[:5]
         self._setup_encryption(symmetric_encryption_keys)
 
+        self.async_lib_config = config
+
         try:
             kwargs['OPTIONS']
         except KeyError:
@@ -56,17 +62,18 @@ class PostgresChannelLayer(BaseChannelLayer):
         self.django_db = DatabaseLayer(logger=logger)
 
     async def get_pool(self):
-        global pool, creating_pool
-        if creating_pool and pool is None:
+        global pool, short_pool, creating_pool
+        if creating_pool and (pool is None or short_pool is None):
             await asyncio.sleep(0.1)
             return await self.get_pool()
 
-        if pool is None:
+        if pool is None or short_pool is None:
             creating_pool = True
-            pool = await aiopg.create_pool(**self.db_params)
+            pool = await aiopg.create_pool(**self.db_params, **self.async_lib_config)
+            short_pool = await aiopg.create_pool(**self.db_params, maxsize=5)
             creating_pool = False
 
-        return pool
+        return pool, short_pool
 
     def _setup_encryption(self, symmetric_encryption_keys):
         # See if we can do encryption if they asked
@@ -102,7 +109,11 @@ class PostgresChannelLayer(BaseChannelLayer):
         assert "__asgi_channel__" not in message
         message = self.serialize(message)
 
-        await self.django_db.send_to_channel([channel], message, self.expiry)
+        _, pool = await self.get_pool()
+        with await pool as conn:
+            await self.django_db.send_to_channel(
+                conn, '', message, self.expiry, channel=channel
+            )
 
     async def _get_message_from_channel(self, channel):
         retrieve_events_sql = f'LISTEN "{channel}";'
@@ -119,7 +130,7 @@ class PostgresChannelLayer(BaseChannelLayer):
                 RETURNING message;
             """
 
-        pool = await self.get_pool()
+        pool, _ = await self.get_pool()
         with await pool as conn:
             cur = await conn.cursor()
             await cur.execute(retrieve_queued_messages_sql, (channel,))
@@ -129,6 +140,7 @@ class PostgresChannelLayer(BaseChannelLayer):
                 # Unlisten and clear pending messages (From other connections) from the queue
                 await cur.execute('UNLISTEN *;')
                 conn._notifies = asyncio.Queue()
+                conn._notifies_proxy = aiopg.utils.ClosableQueue(conn._notifies, conn._loop)
 
                 await cur.execute(retrieve_events_sql)
                 event = await conn.notifies.get()
@@ -158,12 +170,6 @@ class PostgresChannelLayer(BaseChannelLayer):
                 self.client_prefix + '!'
             ), 'Wrong client prefix'
 
-        # Delete expired groups (if enabled) and messages
-        if self.group_expiry > 0:
-            create_task(self.django_db.delete_expired_groups())
-
-        create_task(self.django_db.delete_expired_messages())
-
         return await self._get_message_from_channel(channel)
 
     async def new_channel(self, prefix='specific'):
@@ -191,7 +197,8 @@ class PostgresChannelLayer(BaseChannelLayer):
             'TRUNCATE channels_postgres_message;'
         )
 
-        with await (await self.get_pool()) as conn:
+        _, pool = await self.get_pool()
+        with await pool as conn:
             cur = await conn.cursor()
             await cur.execute(truncate_table_sql)
 
@@ -200,11 +207,14 @@ class PostgresChannelLayer(BaseChannelLayer):
     async def group_add(self, group, channel):
         """Adds the channel name to a group to the Postgres table."""
         group_key = self._group_key(group)
-        await self.django_db.add_channel_to_group(
-            group_key, channel, self.group_expiry
-        )
 
-    async def group_discard(self, group, channel):
+        _, pool = await self.get_pool()
+        with await pool as conn:
+            await self.django_db.add_channel_to_group(
+                conn, group_key, channel, self.group_expiry
+            )
+
+    async def group_discard(self, group, channel, expire=None):
         """
         Removes the channel from the named group if it is in the group;
         does nothing otherwise (does not error)
@@ -217,10 +227,17 @@ class PostgresChannelLayer(BaseChannelLayer):
             'DELETE FROM channels_postgres_groupchannel '
             'WHERE group_key=%s AND channel =%s'
         )
-        with await (await self.get_pool()) as conn:
+        _, pool = await self.get_pool()
+        with await pool as conn:
             cur = await conn.cursor()
             await cur.execute(delete_channel_sql, (group_key, channel))
 
+        # Delete expired groups (if enabled) and messages
+        if self.group_expiry > 0:
+            create_task(self.django_db.delete_expired_groups(self.db_params))
+
+        create_task(self.django_db.delete_expired_messages(self.db_params, expire))
+
     async def group_send(self, group, message):
         """Sends a message to the entire group."""
         assert self.valid_group_name(group), "Group name not valid"
@@ -228,13 +245,12 @@ class PostgresChannelLayer(BaseChannelLayer):
         group_key = self._group_key(group)
         # Retrieve list of all channel names related to the group
         message = self.serialize(message)
-        group_channels = await self.django_db.retrieve_group_channels(
-            group_key
-        )
 
-        await self.django_db.send_to_channel(
-            group_channels, message, self.expiry
-        )
+        _, pool = await self.get_pool()
+        with await pool as conn:
+            await self.django_db.send_to_channel(
+                conn, group_key, message, self.expiry
+            )
 
     def _group_key(self, group):
         """Common function to make the storage key for the group."""
